import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {AboutUsPageCmsRepository} from "./about-us-page-cms.repository";
import {UpdateAboutUsPageCmsDto} from "./dto/update-about-us-page-cms.dto";
import {AboutUsPageCmsEntity} from "./entities/about-us-page-cms.entity";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";
import {DoSpacesService} from "src/modules/SpacesModule/SpacesService/doSpacesService";

@Injectable()
export class AboutUsPageCmsService {
    constructor(
        @InjectRepository(AboutUsPageCmsRepository)
        private readonly repository: AboutUsPageCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateAboutUsPageCmsDto,
        files: {
            heroBackgroundImage: UploadedMulterFileI,
            leaderBackgroundImage: UploadedMulterFileI,
            videoLink: UploadedMulterFileI,
        }
    ): Promise<AboutUsPageCmsEntity | Error> {
        if (files && files.heroBackgroundImage) {
            const heroBackgroundImage: any = await this.doSpaceService.uploadFile(files.heroBackgroundImage[0], "AboutUsPageCms/");
            dto.heroBackgroundImage = heroBackgroundImage;
        }
        if (files && files.leaderBackgroundImage) {
            const leaderBackgroundImage: any = await this.doSpaceService.uploadFile(files.leaderBackgroundImage[0], "AboutUsPageCms/");
            dto.leaderBackgroundImage = leaderBackgroundImage;
        }
        if (files && files.videoLink) {
            const videoLink: any = await this.doSpaceService.uploadFile(files.videoLink[0], "AboutUsPageCms/");
            dto.videoLink = videoLink;
        }
        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<AboutUsPageCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}