import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {Constants} from "src/utils/constants";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";
import {TermsPageCmsService} from "./terms-page-cms.service";
import {UpdateTermsPageCmsDto} from "./dto/update-terms-page-cms.dto";

@Controller({path: "terms-page-cms", version: Constants.API_VERSION_1})

export class TermsPageCmsController {
    constructor(private readonly service: TermsPageCmsService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "termsofUseIcon", maxCount: 1},
            {name: "file", maxCount: 1},
        ]
    ))
    updateOrCreateData(
        @Body() dto: UpdateTermsPageCmsDto,
        @UploadedFiles() files: {
            termsofUseIcon: UploadedMulterFileI
            file: UploadedMulterFileI
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
