import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {FileUploadsService} from 'src/modules/file-uploads/file-uploads.service';
import {ApplicationsIntegrationRepository} from './applications-integration.repository';
import {CreateApplicationsIntegrationDto} from './dto/create-applications-integration.dto';
import {ApplicationsIntegrationEntity} from './entity/applications-integration.entity';
import {UpdateApplicationsIntegrationDto} from './dto/update-applications-integration.dto';
import {DoSpacesService} from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import {UploadedMulterFileI} from 'src/modules/SpacesModule/SpacesService';

@Injectable()
export class ApplicationsIntegrationService {
    constructor(
        @InjectRepository(ApplicationsIntegrationRepository)
        private readonly repository: ApplicationsIntegrationRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpacesService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateApplicationsIntegrationDto,
        files: {
            logo?: UploadedMulterFileI
        }
    ): Promise<ApplicationsIntegrationEntity | Error> {
        if (files && files.logo) {
            const url = await this.doSpacesService.uploadFile(files.logo[0], "ApplicationsIntegration/");
            dto.logo = url.toString();
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateApplicationsIntegrationDto,
        files: {
            logo?: UploadedMulterFileI
        }
    ): Promise<ApplicationsIntegrationEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (files && files.logo) {
            const url = await this.doSpacesService.uploadFile(files.logo[0], "ApplicationsIntegration/");
            dto.logo = url.toString();
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<ApplicationsIntegrationEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<ApplicationsIntegrationEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
