import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from '@nestjs/common';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {Constants} from 'src/utils/constants';
import {HowItWorksSectionService} from './how-it-works-section.service';
import {CreateHowItWorksSectionDto} from './dto/create-how-it-works-section.dto';
import {UpdateHowItWorksSectionDto} from './dto/update-how-it-works-section.dto';
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";

@Controller({path: 'how-it-works-section', version: Constants.API_VERSION_1})
export class HowItWorksSectionController {
    constructor(private readonly service: HowItWorksSectionService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor([
        {name: 'icon', maxCount: 1}
    ]))
    create(
        @Body() dto: CreateHowItWorksSectionDto,
        @UploadedFiles() files: {
            icon?: UploadedMulterFileI,
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor([
        {name: 'icon', maxCount: 1}
    ]))
    update(
        @Param('id') id: string,
        @Body() dto: UpdateHowItWorksSectionDto,
        @UploadedFiles() files: {
            icon?: UploadedMulterFileI,
        }) {
        return this.service.update(id, dto, files);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
