import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateLeadersContentDto} from "./dto/create-leaders-content.dto";
import {UpdateLeadersContentDto} from "./dto/update-leaders-content.dto";
import {LeadersContentEntity} from "./entity/leaders-content.entity";

@EntityRepository(LeadersContentEntity)
export class LeadersContentRepository extends Repository<LeadersContentEntity> {
    async createEntity(dto: CreateLeadersContentDto): Promise<LeadersContentEntity | Error> {
        try {
            const {
                name,
                designation,
                image
            } = dto;
            const entity = this.create({
                name,
                designation,
                image
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<LeadersContentEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateLeadersContentDto): Promise<LeadersContentEntity | Error> {
        try {
            await this.update(id, {
                name: dto.name,
                designation: dto.designation,
                image: dto.image
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<LeadersContentEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(LeadersContentEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}