import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateOrganizationSizeDto} from "./dto/create-organization-size.dto";
import {UpdateOrganizationSizeDto} from "./dto/update-organization-size.dto";
import {OrganizationSizeEntity} from "./entity/organization-size.entity";

@EntityRepository(OrganizationSizeEntity)
export class OrganizationSizeRepository extends Repository<OrganizationSizeEntity> {

    async createEntity(dto: CreateOrganizationSizeDto): Promise<OrganizationSizeEntity | Error> {
        try {
            const {
                name
            } = dto;
            const entity = this.create({
                name
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<OrganizationSizeEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<OrganizationSizeEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateOrganizationSizeDto): Promise<OrganizationSizeEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(OrganizationSizeEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}