import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreatePremiumPartnersContentDto} from "./dto/create-premium-partners-content.dto";
import {UpdatePremiumPartnersContentDto} from "./dto/update-premium-partners-content.dto";
import {PremiumPartnersContentEntity} from "./entity/premium-partners-content.entity";

@EntityRepository(PremiumPartnersContentEntity)
export class PremiumPartnersContentRepository extends Repository<PremiumPartnersContentEntity> {
    async createEntity(dto: CreatePremiumPartnersContentDto): Promise<PremiumPartnersContentEntity | Error> {
        try {
            const {
                image
            } = dto;
            const entity = this.create({
                image
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<PremiumPartnersContentEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdatePremiumPartnersContentDto): Promise<PremiumPartnersContentEntity | Error> {
        try {
            await this.update(id, {
                image: dto.image
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<PremiumPartnersContentEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(PremiumPartnersContentEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}