import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateScheduleTransactionLimitDto} from "./dto/create-schedule-transaction-limit.dto";
import {UpdateScheduleTransactionLimitDto} from "./dto/update-schedule-transaction-limit.dto";
import {ScheduleTransactionLimitEntity} from "./entity/schedule-transaction-limit.entity";

@EntityRepository(ScheduleTransactionLimitEntity)
export class ScheduleTransactionLimitRepository extends Repository<ScheduleTransactionLimitEntity> {

    async createEntity(dto: CreateScheduleTransactionLimitDto): Promise<ScheduleTransactionLimitEntity | Error> {
        try {
            const {
                limit
            } = dto;
            const entity = this.create({
                limit
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ScheduleTransactionLimitEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ScheduleTransactionLimitEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateScheduleTransactionLimitDto): Promise<ScheduleTransactionLimitEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ScheduleTransactionLimitEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}