import { EntityRepository, getConnection, getManager, Repository } from "typeorm";
import { EventScheduleEntity } from "./entities/event-schedule.entity";
import { CreateEventScheduleDto } from "./dto/createEventSchedule.dto";

@EntityRepository(EventScheduleEntity)
export class EventScheduleRepository extends Repository<EventScheduleEntity> {
    async createEntity(dto: CreateEventScheduleDto): Promise<any | Error> {
        try {
            const entity = this.create(
                {
                    eventName: dto.eventName,
                    startTime: dto.startTime,
                    endTime: dto.endTime,
                    description: dto.description,
                    transactionId: dto.transactionId,
                    isModified: false,
                }
            );
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async createTransactionEventEntity(eventObj: any): Promise<any | Error> {
        try {
            const entity = this.create(
                {
                    eventName: eventObj.eventName,
                    startTime: eventObj.startTime,
                    endTime: eventObj.endTime,
                    description: eventObj.description,
                    transactionId: eventObj.transactionId,
                    isModified: eventObj.isModified,
                }
            );
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<EventScheduleEntity[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<EventScheduleEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findScheduleOfUser(id: string, page: number, limit: number): Promise<any | Error> {

        try {
            let offset = 0;
            if (page > 1) {
                offset = (page - 1) * limit;
            }

            const eventSQL = `SELECT es.* FROM event_schedule es 
            LEFT JOIN event_schedule_guest esg 
            ON es.id = esg.eventScheduleId
            WHERE esg.invitedUserId='${id}' OR esg.invitedBy='${id}' 
            AND es.isDeleted=0 GROUP BY es.id LIMIT ${limit} OFFSET ${offset};`;

            const eventCountSQL = `SELECT count(distinct es.id) count FROM event_schedule es 
            LEFT JOIN event_schedule_guest esg 
            ON es.id = esg.eventScheduleId
            WHERE esg.invitedUserId='${id}' OR esg.invitedBy='${id}' 
            AND es.isDeleted=0;`;

            const eventListCount = await this.manager.query(eventCountSQL);
            const eventList = await this.manager.query(eventSQL);
            const eventListIds = eventList.map((item) => `'${item.id}'`).join(',');

            const totalPages = Math.ceil(eventListCount?.[0]?.count / limit);
            const nextPage = page < totalPages ? page + 1 : null;
            const previousPage = page > 1 ? page - 1 : null;

            if (eventListIds) {
                const guestSQL = `SELECT esg.* FROM event_schedule_guest esg 
                    WHERE esg.eventScheduleId IN(${eventListIds}) AND esg.isDeleted=0;`;

                const guestList = await this.manager.query(guestSQL);

                const guestListMap = new Map<string, any[]>();

                for (const guest of guestList) {
                    if (guestListMap.has(guest.eventScheduleId)) {
                        const data = guestListMap.get(guest.eventScheduleId);
                        guestListMap.set(guest.eventScheduleId, [...data, guest]);
                    } else {
                        guestListMap.set(guest.eventScheduleId, [guest]);
                    }
                }


                const eventListData = eventList.map((data) => {
                    const guestList = guestListMap.get(data.id);
                    return {
                        ...data,
                        guestList,
                    }
                })

                return {
                    events: eventListData,
                    totalCount: eventListCount?.[0]?.count,
                    totalPages,
                    nextPage,
                    previousPage
                }
            } else {
                return {
                    events: [],
                    totalCount: eventListCount?.[0]?.count,
                    totalPages,
                    nextPage,
                    previousPage
                }
            }


        } catch (e) {
            console.log(e);
            return e;
        }

    }

    async findScheduleAllOfUser(id: string): Promise<any | Error> {

        try {

            const eventSQL = `SELECT es.* FROM event_schedule es 
            LEFT JOIN event_schedule_guest esg 
            ON es.id = esg.eventScheduleId
            WHERE esg.invitedUserId='${id}' OR esg.invitedBy='${id}' 
            AND es.isDeleted=0 GROUP BY es.id;`;

            const eventCountSQL = `SELECT count(distinct es.id) count FROM event_schedule es 
            LEFT JOIN event_schedule_guest esg 
            ON es.id = esg.eventScheduleId
            WHERE esg.invitedUserId='${id}' OR esg.invitedBy='${id}' 
            AND es.isDeleted=0;`;

            const eventListCount = await this.manager.query(eventCountSQL);
            const eventList = await this.manager.query(eventSQL);
            const eventListIds = eventList.map((item) => `'${item.id}'`).join(',');

            if (eventListIds) {
                const guestSQL = `SELECT esg.* FROM event_schedule_guest esg 
                    WHERE esg.eventScheduleId IN(${eventListIds}) AND esg.isDeleted=0;`;

                const guestList = await this.manager.query(guestSQL);

                const guestListMap = new Map<string, any[]>();

                for (const guest of guestList) {
                    if (guestListMap.has(guest.eventScheduleId)) {
                        const data = guestListMap.get(guest.eventScheduleId);
                        guestListMap.set(guest.eventScheduleId, [...data, guest]);
                    } else {
                        guestListMap.set(guest.eventScheduleId, [guest]);
                    }
                }


                const eventListData = eventList.map((data) => {
                    const guestList = guestListMap.get(data.id);
                    return {
                        ...data,
                        guestList,
                    }
                })

                return {
                    events: eventListData,
                }
            } else {
                return {
                    events: []
                }
            }


        } catch (e) {
            console.log(e);
            return e;
        }

    }

    async updateEntity(id: string, dto: any): Promise<EventScheduleEntity | Error> {
        try {
            const updateDto = {
                eventName: dto.eventName,
                startTime: dto.startTime,
                endTime: dto.endTime,
                description: dto.description,
                transactionId: dto.transactionId,
            }
            await this.update(id, { ...updateDto });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateTransactionEventEntity(id: string, eventObj: any): Promise<EventScheduleEntity | Error> {
        try {
            const updateDto = {
                eventName: eventObj.eventName,
                startTime: eventObj.startTime,
                endTime: eventObj.endTime,
                description: eventObj.description,
                transactionId: eventObj.transactionId,
            }
            await this.update(id, { ...updateDto });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(EventScheduleEntity)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async deleteTimelineEntity(eventName: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(EventScheduleEntity)
                .set({ isDeleted: true })
                .where("eventName = :eventName", { eventName: eventName })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}