import {EntityRepository, In, LessThanOrEqual, MoreThanOrEqual, Not, Repository} from "typeorm";
import {SingleMessageDto} from "./dto/single-message.dto";
import {Message} from "./entities/message.entity";
import {OneToOneMessageDto} from "./dto/one-to-one-message.dto";
import {GroupMessageDto} from "./dto/group-message.dto";
import {AllGroupMessageDto} from "./dto/all-group-message.dto";
import { NotificationMessageDto } from "./dto/notification-message.dto";

@EntityRepository(Message)
export class MessageRepository extends Repository<Message> {
    async createEntity(dto: SingleMessageDto): Promise<Message | Error> {
        try {
            const entity = this.create({...dto});
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findAllOneToOne(dto: OneToOneMessageDto): Promise<Message[] | Error> {
        const senderToReceive = {senderId: dto.senderId, receiverId: dto.receiverId};
        const receiverToSender = {receiverId: dto.senderId, senderId: dto.receiverId}
        try {
            return await this.find({where: [senderToReceive, receiverToSender], order: {createdAt: 'ASC'}})
        } catch (error) {
            throw error;
        }
    }

    async createEntityForGroup(dto: GroupMessageDto) {
        try {
            const entity = this.create({...dto});
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findAllGroupMessage(dto: AllGroupMessageDto): Promise<Message[] | Error> {
        try {
            return await this.find({where: {transactionId: dto.transactionId}, order: {createdAt: 'ASC'}})
        } catch (error) {
            throw error;
        }
    }

    async findNotificationsMessageByUserId(dto: NotificationMessageDto) {
        try {
            const partyQuery = `SELECT transactionId FROM party where userId = '${dto.userId}'`;
            const parties = await this.manager.query(partyQuery);
            const transactionIds = parties.map((x: any) => `'${x.transactionId}'`);
            
            const message = await this.find({
                where: [
                    { createdAt: MoreThanOrEqual(new Date(dto.notificationTime)), receiverId: dto.userId },
                    { createdAt: MoreThanOrEqual(new Date(dto.notificationTime)), transactionId: In(transactionIds), senderId: Not(dto.userId) }
                ]
            });

            if(message.length == 0) return message;

            const userIds = [...new Set(message.map(x => `'${x.senderId}'`))];

            const userQuery = `select * from users where id in (${userIds.join(',')})`;
            const users = await this.query(userQuery);
            const userMap = new Map<string, any>();
            for (const user of users) {
                userMap.set(user.id, user);
            }

            return message.map(x => {
                const user = userMap.get(x.senderId);
                return {
                    ...x,
                    image: user.image
                }
            });
        } catch (error) {
            throw error;
        }
    }
}