import {Body, Controller, Delete, Get, Param, Patch, Post, Req, UseGuards} from '@nestjs/common';
import {Constants} from 'src/utils/constants';
import {PartyService} from './party.service';
import {CreatePartyDto} from './dto/create-party.dto';
import {UpdatePartyDto} from './dto/update-party.dto';
import {AuthGuard} from "../../core/guards/auth.guard";

@Controller({path: 'party', version: Constants.API_VERSION_1})
export class PartyController {
    constructor(private readonly service: PartyService) {
    }

    @Post()
    @UseGuards(AuthGuard)
    create(@Body() dto: CreatePartyDto, @Req() request: any) {
        return this.service.create(dto, request.user);
    }

    @Get()
    @UseGuards(AuthGuard)
    findAll(@Req() request: any) {
        return this.service.findAll(request.user);
    }

    @Get('all-party-by-user')
    @UseGuards(AuthGuard)
    getAllPartyByUserId(@Req() request: any) {
        return this.service.getAllPartyByUserId(request.user.id);
    }

    @Get(':id')
    @UseGuards(AuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdatePartyDto, @Req() request: any) {
        return this.service.update(id, dto, request.user);
    }

    @Delete(':id')
    @UseGuards(AuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
