import { EntityRepository, getConnection, Repository } from "typeorm";
import { TransactionDocumentsEntity } from "./entity/transaction-documents.entity";
import { UpdateTransactionDocumentsDto } from "./dto/update-transaction-documents.dto";

@EntityRepository(TransactionDocumentsEntity)
export class TransactionDocumentsRepository extends Repository<TransactionDocumentsEntity> {
    async createEntity(dto: any) {
        try {
            const entity = this.create(
                dto
            );
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<TransactionDocumentsEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateTransactionDocumentsDto): Promise<TransactionDocumentsEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<TransactionDocumentsEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                order: {
                    createdAt: "ASC"
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(TransactionDocumentsEntity)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            return updateResult.affected > 0;

        } catch (e) {
            return e;
        }
    }
}