import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    Query,
    Req,
    Request,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import {UsersService} from "./users.service";
import {CreateUserDto} from "./dto/create-user.dto";
import {UpdateUserDto} from "./dto/update-user.dto";
import {Constants} from "src/utils/constants";
import {AuthGuard} from "src/core/guards/auth.guard";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {UploadedMulterFileI} from "../SpacesModule/SpacesService";

@Controller({
    path: "users",
    version: Constants.API_VERSION_1
})
export class UsersController {
    constructor(private readonly usersService: UsersService) {
    }

    @Post()
    create(@Body() dto: CreateUserDto) {
        return this.usersService.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll(@Query() userType?: string) {
        return this.usersService.findAll(userType);
    }

    @Get('common-transaction-user')
    @UseGuards(AuthGuard)
    findAllCommonTransactionUser(@Req() req: any) {
        return this.usersService.findAllCommonTransactionUser(req.user.id);
    }

    @UseGuards(AuthGuard)
    @Get('current-transaction-user')
    findCurrentTransactionUser(@Req() req: any, @Query('userType') userType?: string, @Query('search') search?: string) {
        return this.usersService.findCurrentTransactionUser(req.user.id, userType, search);
    }

    @Patch("/update-user-profile")
    @UseGuards(AuthGuard)
    @UseInterceptors(FileFieldsInterceptor([{name: "image", maxCount: 1}]))
    updateUserProfile(
        @Req() request: any,
        @Body() dto: UpdateUserDto,
        @UploadedFiles() files: { image: UploadedMulterFileI }
    ) {
        return this.usersService.updateUserProfile(request, dto, files);
    }

    @Get(":id")
    @UseGuards(AuthGuard)
    findOne(@Param("id") id: string) {
        return this.usersService.findOne(id);
    }

    @Patch("/status-update/:id")
    @UseGuards(AdminAuthGuard)
    userStatusUpdate(@Param("id") id: string, @Body() dto: any) {
        return this.usersService.userStatusUpdate(id, dto);
    }

    @Patch(":id")
    @UseGuards(AdminAuthGuard)
    update(@Param("id") id: string, @Body() dto: UpdateUserDto) {
        return this.usersService.update(id, dto);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    remove(@Param("id") id: string) {
        return this.usersService.remove(id);
    }
}
